/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package us.levk.remote.drmaa.client;

import java.io.File;
import java.net.URL;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.HierarchicalConfiguration;
import org.apache.commons.configuration.XMLConfiguration;
import org.apache.log4j.Logger;

/**
 * <p>Configuration of the client.</p>
 * 
 * <p>The client will look for configuration in 3 successive locations:
 * <li>First if REMOTE_DRMAA_CLIENT_XML environment variable is defined to an existing file it will be used</li>
 * <li>If that fails the client will look for file <code>.remote-drmaa-client.xml</code> in the user's home folder</li>
 * <li>If that fails the client will look for the same file name in the java home folder</li>
 * <li>If that fails a warning is printed to logs, client configuration is optional</li></p>
 * 
 * <p>The configuration file should be of this form:
 * <pre>
 * <client>
 *   <servers>
 *     <server>
 *       <url>http://mighty-mouse:8080/remote-drmaa-server/xmlrpc</url>
 *       <!--contact></contact-->
 *       <username>foo</username>
 *       <password>bar</password>
 *     </server>
 *   </servers>
 * </client>
 * </pre>
 * Here you may specify contact and authentication information for each server by its URL. Please note that specifying
 * contact information will override any presets on the server, and specifying an empty contact is not the same as not
 * specifying it.</p>
 * 
 * @author levk
 *
 */
public abstract class Configuration {

  /**
   * Little instance to pass credentials
   * 
   * @author levk
   *
   */
  public interface Credential {
    public String getUserName ();
    public String getPassword ();
  }

  private static final transient Logger LOG = Logger.getLogger (Configuration.class);

  private static final transient Configuration INSTANCE = new Configuration () {

    private final transient XMLConfiguration CONFIGURATION;

    {
      XMLConfiguration config = null;
      try {
        config = new XMLConfiguration (System.getenv ("REMOTE_DRMAA_CLIENT_XML"));
        LOG.info ("Loaded configuration from " + System.getenv ("REMOTE_DRMAA_CLIENT_XML"));
      } catch (ConfigurationException e1) {
        try {
          config = new XMLConfiguration (System.getProperty ("user.home") + File.separator + ".remote-drmaa-client.xml");
          LOG.info ("Loaded configuration from " + System.getProperty ("user.home") + File.separator + ".remote-drmaa-client.xml");
        } catch (ConfigurationException e2) {
          try {
            config = new XMLConfiguration (System.getProperty ("java.home") + File.separator + ".remote-drmaa-client.xml");
            LOG.info ("Loaded configuration from " + System.getProperty ("java.home") + File.separator + ".remote-drmaa-client.xml");
          } catch (ConfigurationException e3) {
            config = new XMLConfiguration ();
            LOG.warn ("Unable to load configuration");
          }
        }
      }
      CONFIGURATION = config;
    }

    public Credential getCredential (URL url) {
      for (final HierarchicalConfiguration c : CONFIGURATION.configurationsAt ("servers.server"))
        if (url.toString ().equals (c.getString ("url")))
          return new Credential () {
            public String getUserName () { return c.getString ("username"); }
            public String getPassword () { return c.getString ("password"); }
          };
      return null;
    }

    public String getContact (URL url) {
      for (HierarchicalConfiguration c : CONFIGURATION.configurationsAt ("servers.server"))
        if (c.getString ("url").equals (url.toString ())) return c.getString ("contact");
      return null;
    }
  };

  public static Configuration getConfiguration () { return INSTANCE; }

  /**
   * Gets credential information for a server by url or null if not defined for the specified url
   * 
   * @param url
   * @return
   */
  public abstract Credential getCredential (URL url);

  /**
   * Gets override contact string or null if there isn't one defined for the url specified
   * 
   * @param url
   * @return
   */
  public abstract String getContact (URL url);
}
